package cn.jiguang.imui.messagelist.themes.black;

import android.app.Notification;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.ImageButton;
import android.widget.ProgressBar;
import android.widget.TextView;

import java.text.ParseException;
import java.util.Date;
import java.util.Locale;

import cn.jiguang.imui.commons.models.IMessage;
import cn.jiguang.imui.messagelist.BuildConfig;
import cn.jiguang.imui.messagelist.R;
import cn.jiguang.imui.messagelist.model.RCTMessage;
import cn.jiguang.imui.messages.BaseMessageViewHolder;
import cn.jiguang.imui.messages.MessageListStyle;
import cn.jiguang.imui.messages.MsgListAdapter;
import cn.jiguang.imui.view.RoundImageView;

import java.text.SimpleDateFormat;

public class BlackTxtViewHolder<MESSAGE extends IMessage> extends  BaseMessageViewHolder<MESSAGE> implements MsgListAdapter.DefaultMessageViewHolder {
    TextView mMsgTv = null;
    private TextView mDateTv = null;
    private TextView mDisplayNameTv = null;
    private RoundImageView mAvatarIv = null;
    private ImageButton mResendIb = null;
    private ProgressBar mSendingPb = null;

    private Boolean mIsSender;

    //    BlackTxtViewHolder(View itemView, Boolean mIsSender) {
//
//    }
    public BlackTxtViewHolder(View itemView, Boolean isSender) {
        super(itemView);
        this.mIsSender = isSender;
        mMsgTv = itemView.findViewById(R.id.aurora_tv_msgitem_message);
        mDateTv = itemView.findViewById(R.id.aurora_tv_msgitem_date);
        if (mIsSender) {
            mDisplayNameTv = itemView.findViewById(R.id.aurora_tv_msgitem_sender_display_name);
            mSendingPb = itemView.findViewById(R.id.aurora_pb_msgitem_sending);
            mResendIb = itemView.findViewById(R.id.aurora_ib_msgitem_resend);
        } else {
            mDisplayNameTv = itemView.findViewById(R.id.aurora_tv_msgitem_receiver_display_name);
        }
        mAvatarIv = itemView.findViewById(R.id.aurora_iv_msgitem_avatar);
    }

    @Override
    public void applyStyle(MessageListStyle style) {
        mMsgTv.setMaxWidth((int) (style.getWindowWidth() * style.getBubbleMaxWidth()));
        mMsgTv.setLineSpacing((float) style.getLineSpacingExtra(), style.getLineSpacingMultiplier());
        if (mIsSender) {
            mMsgTv.setBackground(style.getSendBubbleDrawable());
            mMsgTv.setTextColor(style.getSendBubbleTextColor());
            mMsgTv.setTextSize(style.getSendBubbleTextSize());
            mMsgTv.setPadding(style.getSendBubblePaddingLeft(),
                    style.getSendBubblePaddingTop(),
                    style.getSendBubblePaddingRight(),
                    style.getSendBubblePaddingBottom());
            if (style.getSendingProgressDrawable() != null) {
                mSendingPb.setProgressDrawable(style.getSendingProgressDrawable());
            }
            if (style.getSendingIndeterminateDrawable() != null) {
                mSendingPb.setIndeterminateDrawable(style.getSendingIndeterminateDrawable());
            }
            if (style.getShowSenderDisplayName()) {
                mDisplayNameTv.setVisibility(View.VISIBLE);
            } else {
                mDisplayNameTv.setVisibility(View.GONE);
            }
        } else {
            mMsgTv.setBackground(style.getReceiveBubbleDrawable());
            mMsgTv.setTextColor(style.getReceiveBubbleTextColor());
            mMsgTv.setTextSize(style.getReceiveBubbleTextSize());
            mMsgTv.setPadding(style.getReceiveBubblePaddingLeft(),
                    style.getReceiveBubblePaddingTop(),
                    style.getReceiveBubblePaddingRight(),
                    style.getReceiveBubblePaddingBottom());
            if (style.getShowReceiverDisplayName()) {
                mDisplayNameTv.setVisibility(View.VISIBLE);
            } else {
                mDisplayNameTv.setVisibility(View.GONE);
            }
        }
        mDateTv.setTextSize(style.getDateTextSize());
        mDateTv.setTextColor(style.getDateTextColor());

        mAvatarIv.getLayoutParams().width = style.getAvatarWidth();
        mAvatarIv.getLayoutParams().height = style.getAvatarHeight();
        mAvatarIv.setBorderRadius(style.getAvatarRadius());
    }

    @Override
    public void onBind(final MESSAGE message) {
//        final MESSAGE message = (MESSAGE) obj;
        mMsgTv.setText(message.getText());
        if (message.getTimeString() != null) {
            mDateTv.setText(message.getTimeString());
        }
        boolean isAvatarExists = message.getFromUser().getAvatarFilePath() != null && !message.getFromUser().getAvatarFilePath().isEmpty();
        if (isAvatarExists && mImageLoader != null) {
            if (mPosition + 1 < mData.size()) {
                MsgListAdapter.Wrapper lastWrapper = (MsgListAdapter.Wrapper) mData.get(mPosition + 1);
                if (lastWrapper.getItem() instanceof IMessage) {
                    RCTMessage lastMsg = (RCTMessage) lastWrapper.getItem();
                    // config time string
                    if (!TextUtils.isEmpty(message.getTimeString()) && !TextUtils.isEmpty(lastMsg.getTimeString())) {
                        SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm", Locale.getDefault());
                        try {
                            Date date1 = dateFormat.parse(message.getTimeString());
                            Date date2 = dateFormat.parse(lastMsg.getTimeString());
                            if (date1.getTime() - date2.getTime() > 1 * 60 * 1000) {
                                mDateTv.setVisibility(View.VISIBLE);
                            } else {
                                mDateTv.setVisibility(View.GONE);
                            }
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                    }

                    if (lastMsg.getFromUser().getId() == message.getFromUser().getId()) {
                        mAvatarIv.setVisibility(View.GONE);
                    } else {
                        mAvatarIv.setVisibility(View.VISIBLE);
                        mImageLoader.loadAvatarImage(mAvatarIv, message.getFromUser().getAvatarFilePath());
                    }
                }
            } else {
                mAvatarIv.setVisibility(View.VISIBLE);
                mImageLoader.loadAvatarImage(mAvatarIv, message.getFromUser().getAvatarFilePath());
            }
        } else if (mImageLoader == null) {
            mAvatarIv.setVisibility(View.GONE);
        }
        if (mDisplayNameTv.getVisibility() == View.VISIBLE) {
            mDisplayNameTv.setText(message.getFromUser().getDisplayName());
        }
        if (mIsSender) {
            switch (message.getMessageStatus()) {
                case SEND_GOING:
                    mSendingPb.setVisibility(View.VISIBLE);
                    mResendIb.setVisibility(View.GONE);
                    break;
                case SEND_FAILED:
                    mSendingPb.setVisibility(View.GONE);
                    mResendIb.setVisibility(View.VISIBLE);
//                    final RCTMessage _rctMessage = (RCTMessage) message;
                    mResendIb.setOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View view) {
                            if (mMsgStatusViewClickListener != null) {
                                mMsgStatusViewClickListener.onStatusViewClick(message);
                            }
                        }
                    });
                    break;
                default:
                    mSendingPb.setVisibility(View.GONE);
                    mResendIb.setVisibility(View.GONE);
                    break;
            }
        }

        mMsgTv.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (mMsgClickListener != null) {
                    mMsgClickListener.onMessageClick(message);
                }
            }
        });

        mMsgTv.setOnLongClickListener(new View.OnLongClickListener() {
            @Override
            public boolean onLongClick(View view) {
                if (mMsgLongClickListener != null) {
                    mMsgLongClickListener.onMessageLongClick(view, message);
                } else {
                    if (BuildConfig.DEBUG) {
                        Log.w("MsgListAdapter", "Didn't set long click listener! Drop event.");
                    }
                }
                return true;
            }
        });

        mAvatarIv.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (mAvatarClickListener != null) {
                    mAvatarClickListener.onAvatarClick(message);
                }
            }
        });
    }
}
