'use strict';

import React from 'react';
import ReactNative from 'react-native';
import PropTypes from 'prop-types';
import { ViewPropTypes } from 'react-native';

var {
	Component,
} = React;

var {
	StyleSheet,
	View,
	Dimensions,
	requireNativeComponent,
	UIManager,
	findNodeHandle, NativeModules
} = ReactNative;

const AuroraIMUIModule = NativeModules.AuroraIMUIModule;
const CHAT_INPUT = "chat_input";

export default class ChatInput extends Component {

	constructor(props) {
		super(props);
		this._onSendText = this._onSendText.bind(this);
		this._onSendFiles = this._onSendFiles.bind(this);
		this._takePicture = this._takePicture.bind(this);
		this._startVideoRecord = this._startVideoRecord.bind(this);
		this._finishVideoRecord = this._finishVideoRecord.bind(this);
		this._cancelVideoRecord = this._cancelVideoRecord.bind(this);
		this._onStartRecordVoice = this._onStartRecordVoice.bind(this);
		this._onFinishRecordVoice = this._onFinishRecordVoice.bind(this);
		this._onCancelRecordVoice = this._onCancelRecordVoice.bind(this);
		this._onSwitchToMicrophoneMode = this._onSwitchToMicrophoneMode.bind(this);
		this._onSwitchGalleryMode = this._onSwitchGalleryMode.bind(this);
		this._onSwitchToCameraMode = this._onSwitchToCameraMode.bind(this);
		this._onSwitchToEmojiMode = this._onSwitchToEmojiMode.bind(this);
		this._onTouchEditText = this._onTouchEditText.bind(this);
		this._onTouchMore = this._onTouchMore.bind(this);
		this._onFullScreen = this._onFullScreen.bind(this);
		this._onRecoverScreen = this._onRecoverScreen.bind(this);
		this._onSizeChange = this._onSizeChange.bind(this);
		this._onClickSelectAlbum = this._onClickSelectAlbum.bind(this);
		this._closeCamera = this._closeCamera.bind(this);
		this._switchCameraMode = this._switchCameraMode.bind(this);
		this._onSwitchInputToolsBar = this._onSwitchInputToolsBar.bind(this);
	}

	_onSendText(event: Event) {
		if (!this.props.onSendText) {
			return;
		}
		this.props.onSendText(event.nativeEvent.text);
	}

	_onSendFiles(event: Event) {
		if (!this.props.onSendGalleryFiles) {
			return;
		}
		this.props.onSendGalleryFiles(event.nativeEvent.mediaFiles);
	}

	_takePicture(event: Event) {
		if (!this.props.onTakePicture) {
			return;
		}
		this.props.onTakePicture(event.nativeEvent);
	}

	_startVideoRecord() {
		if (!this.props.onStartRecordVideo) {
			return;
		}
		this.props.onStartRecordVideo();
	}

	_finishVideoRecord(event: Event) {
		if (!this.props.onFinishRecordVideo) {
			return;
		}
		this.props.onFinishRecordVideo(event.nativeEvent);
	}

	_cancelVideoRecord() {
		if (!this.props.onCancelRecordVideo) {
			return;
		}
		this.props.onCancelRecordVideo();
	}

	_onStartRecordVoice() {
		if (!this.props.onStartRecordVoice) {
			return;
		}
		this.props.onStartRecordVoice();
	}

	_onFinishRecordVoice(event: Event) {
		if (!this.props.onFinishRecordVoice) {
			return;
		}
		this.props.onFinishRecordVoice(event.nativeEvent.mediaPath, event.nativeEvent.duration);
	}

	_onCancelRecordVoice() {
		if (!this.props.onCancelRecordVoice) {
			return;
		}
		this.props.onCancelRecordVoice();
	}

	_onSwitchToMicrophoneMode() {
		if (!this.props.onSwitchToMicrophoneMode) {
			return;
		}
		this.props.onSwitchToMicrophoneMode();
	}

	_onSwitchGalleryMode() {
		if (!this.props.onSwitchToGalleryMode) {
			return;
		}
		this.props.onSwitchToGalleryMode();
	}

	_onSwitchToCameraMode() {
		if (!this.props.onSwitchToCameraMode) {
			return;
		}
		this.props.onSwitchToCameraMode();
	}

	_onSwitchToEmojiMode() {
		if (!this.props.onSwitchToEmojiMode) {
			return;
		}
		this.props.onSwitchToEmojiMode();
	}

	_onTouchEditText() {
		if (!this.props.onTouchEditText) {
			return;
		}
		this.props.onTouchEditText();
	}

	_onFullScreen() {
		if (!this.props.onFullScreen) {
			return;
		}
		this.props.onFullScreen();
	}

	_onRecoverScreen() {
		if (!this.props.onRecoverScreen) {
			return;
		}
		this.props.onRecoverScreen();
	}

	_onSizeChange(event: Event) {
		if (!this.props.onSizeChange) {
			return;
		}
		this.props.onSizeChange({ width: Dimensions.get('window').width, inputToolsBarHeight: event.nativeEvent.inputToolsBarHeight, inputBarHeight: event.nativeEvent.inputBarHeight, inputViewHeight: event.nativeEvent.inputViewHeight });
	}

	_onClickSelectAlbum(event: Event) {
		if (!this.props.onClickSelectAlbum) {
			return;
		}
		this.props.onClickSelectAlbum();
	}

	_onSwitchInputToolsBar(event: Event) {
		if (!this.props.onSwitchInputToolsBar) {
			return;
		}
		this.props.onSwitchInputToolsBar({ inputToolsBarSwitchType: event.nativeEvent.inputToolsBarSwitchType });
	}

	_closeCamera(event: Event) {
		if (!this.props.closeCamera) {
			return;
		}
		this.props.closeCamera();
	}

	_switchCameraMode(event: Event) {
		if (!this.props.switchCameraMode) {
			return;
		}
		this.props.switchCameraMode(event.nativeEvent.isRecordVideoMode);
	}

	setMenuContainerHeight(height) {
		UIManager.dispatchViewManagerCommand(findNodeHandle(this.refs[CHAT_INPUT]), 99, [height]);
	}

	closeSoftInput() {
		UIManager.dispatchViewManagerCommand(findNodeHandle(this.refs[CHAT_INPUT]), 100, null);
	}

	getInputText() {
		UIManager.dispatchViewManagerCommand(findNodeHandle(this.refs[CHAT_INPUT]), 101, null);
	}


	inputTextFocus() {
		UIManager.dispatchViewManagerCommand(findNodeHandle(this.refs[CHAT_INPUT]), 103, null);
	}


	showMenu(flag) {
		UIManager.dispatchViewManagerCommand(findNodeHandle(this.refs[CHAT_INPUT]), 102, [flag]);
	}

	/**
	 * 输入框整个VIEW失去焦点，输入框工具栏会自动关闭
	 */
	inputViewOnBlur() {
		// alert(AuroraIMUIController);//.inputViewOnBlur);
		// AuroraIController.inputViewOnBlur(this.refs[CHAT_INPUT]._nativeTag);
		AuroraIMUIModule.inputViewOnBlur(this.refs[CHAT_INPUT]._nativeTag);
	}

	/**
	 * 判断输入框工具栏是否打开
	 */
	/**
	 * 判断输入框工具栏是否打开
	 * @param {*} fun  1：显示、0：未显示、-1：未获取到输入组件对象
	 */
	isShowInputToolsBar(fun) {
		AuroraIMUIModule.isShowInputToolsBar(this.refs[CHAT_INPUT]._nativeTag, fun);
	}

	_onTouchMore() {
		if (!this.props.onTouchMore) {
			return;
		}
		this.props.onTouchMore();
	}

	render() {
		return (
			<RCTChatInput
				ref={CHAT_INPUT}
				{...this.props}
				onSendText={this._onSendText}
				onSendGalleryFiles={this._onSendFiles}
				onTakePicture={this._takePicture}
				onStartRecordVideo={this._startVideoRecord}
				onFinishRecordVideo={this._finishVideoRecord}
				onCancelRecordVideo={this._cancelVideoRecord}
				onStartRecordVoice={this._onStartRecordVoice}
				onFinishRecordVoice={this._onFinishRecordVoice}
				onCancelRecordVoice={this._onCancelRecordVoice}
				onSwitchToMicrophoneMode={this._onSwitchToMicrophoneMode}
				onSwitchToGalleryMode={this._onSwitchGalleryMode}
				onSwitchToCameraMode={this._onSwitchToCameraMode}
				onTouchEditText={this._onTouchEditText}
				onTouchMore={this._onTouchMore}
				onFullScreen={this._onFullScreen}
				onRecoverScreen={this._onRecoverScreen}
				onSizeChange={this._onSizeChange}
				onClickSelectAlbum={this._onClickSelectAlbum}
				closeCamera={this._closeCamera}
				switchCameraMode={this._switchCameraMode}
				onSwitchInputToolsBar={this._onSwitchInputToolsBar}
			/>
		);
	}

}

ChatInput.propTypes = {
	chatInputBackgroundColor: PropTypes.string,
	menuContainerHeight: PropTypes.number,
	isDismissMenuContainer: PropTypes.bool,
	onSendText: PropTypes.func,
	onSendGalleryFiles: PropTypes.func,
	onTakePicture: PropTypes.func,
	onStartRecordVideo: PropTypes.func,
	onFinishRecordVideo: PropTypes.func,
	onCancelRecordVideo: PropTypes.func,
	onStartRecordVoice: PropTypes.func,
	onFinishRecordVoice: PropTypes.func,
	onCancelRecordVoice: PropTypes.func,
	onSwitchToMicrophoneMode: PropTypes.func,
	onSwitchToGalleryMode: PropTypes.func,
	onSwitchToCameraMode: PropTypes.func,
	onSwitchToEmojiMode: PropTypes.func,
	onTouchEditText: PropTypes.func,
	onFullScreen: PropTypes.func,
	onRecoverScreen: PropTypes.func,
	onSizeChange: PropTypes.func,
	closeCamera: PropTypes.func,
	switchCameraMode: PropTypes.func,
	onClickSelectAlbum: PropTypes.func,
	showSelectAlbumBtn: PropTypes.bool,
	showRecordVideoBtn: PropTypes.bool,
	inputPadding: PropTypes.object,
	inputTextColor: PropTypes.string,
	inputTextSize: PropTypes.number,
	inputTextLineHeight: PropTypes.number,
	hideCameraButton: PropTypes.bool,
	hideVoiceButton: PropTypes.bool,
	hideEmojiButton: PropTypes.bool,
	hidePhotoButton: PropTypes.bool,
	customLayoutItems: PropTypes.object,
	inputContainerMargin: PropTypes.object,
	inputContainerBackgroundColor: PropTypes.string,
	inputPanelBackgroundColor: PropTypes.string,
	inputTextBackgroundColor: PropTypes.string,
	inputBarMarginLeft: PropTypes.number,
	inputBarMarginRight: PropTypes.number,
	onTouchMore: PropTypes.func,
	onSwitchInputToolsBar: PropTypes.func,
	inputTextHeight: PropTypes.number,
	inputTextMaxLength: PropTypes.number,
	inputMenuMoreBgColor: PropTypes.string,
	inputMenuSendBgColor: PropTypes.string,
	...ViewPropTypes
};

var RCTChatInput = requireNativeComponent('RCTChatInput', ChatInput);